<?php
# rrplacenameformat_lib.php - Include file installed by the Placename Format mod.
# Contains functions used by programs that format USA placenames or that use 
# formatted names to compute place levels or geocodes and the like.
$tngModNames['Placename Format']['rrplacenameformat_lib.php'] = '4'; #Register the mod

#############################################
#Format a placename.
#Given a PlaceName, figure out if we are formatting placename, and, if we are, then break it into
#comma-parts and figure out if it is a USA placename.
#Then call formatPlaceName2 to do the actual formatting.
function formatPlaceName1($placename) {
	global $placeOptions;
	$newPlaceName = $originalPlaceName = $placename;
	try {
		#If there are supposed to be spaces between places, add a space between double commas.
		if (!$placeOptions['compressPlaces']) {
			$placename = str_replace(',,', ', ,', $placename);
			$placename = str_replace(',,', ', ,', $placename);
			$placename = str_replace(',,', ', ,', $placename);
		}

		//Separate the placename into comma-parts
		$commaParts = array_Map(trim,explode(",", $placename));
		$nparts = count($commaParts);
		# Check for USA places
		# Look in the last comma-part for a USA variant, a state name, or a state abbreviation.
		# If we find one, call the Placename Formatting function.
		$lastpart = $commaParts[$nparts-1];
		$lastpartUC = strtoupper($lastpart);
		if (isset($placeOptions['usaPossibleValues'][$lastpartUC])) {
			#The placename ends with a USA variant.
			#If it consists only of a USA variant, change it to the desired form, and return. 
			if ($nparts == 1) {
				#Replace the original reprentation of USA string with the representation of USA
				#that is saved in the option usaTextAlone.
				if ($placeOptions['usaTextAlone'])
					return array('placename'=>$placeOptions['usaTextAlone']); #### Return #####
				else #There's no preferred USA form; use the original USA representation.
					return array('placename'=>$lastpart); ####### Return ###### 
			}
			#Remove the USA part from the array. It'll go back in at the end if usaTextWith is non-empty.
			#The option usaTextWith is used for placenames that have lower-ordered components, whereas
			#usaTextAlone was for a placename that consists only of the country. 
			unset($commaParts[$nparts-1]);
			$nparts--; #We have one fewer commaPart
			$lastpart = $commaParts[$nparts-1]; #The last part is now the state name or abbreviation
			#We need to know whether the state is a full name or abbreviation before calling the Placename Formatting function
			if (isset($placeOptions['stateNames'][$lastpart])) {
				#This placename has a state name
				$newPlaceInfo = formatPlaceName2( $lastpart, $placeOptions['stateNames'][$lastpart], $commaParts);
			}
			elseif (isset($placeOptions['stateAbs'][$lastpart])) {
				#This placename has a state abbreviation
				$newPlaceInfo = formatPlaceName2($placeOptions['stateAbs'][$lastpart], $lastpart, $commaParts);
			} else { #The placename ends with a USA variant, but doesn't include a valid state name or abbreviation.#
				$newPlaceInfo = formatPlaceName2($lastpart, $lastpart, $commaParts);
			}
		}
		elseif (isset($placeOptions['stateNames'][$lastpart])) {
			#This placename ends with a state name
			$newPlaceInfo = formatPlaceName2( $lastpart, $placeOptions['stateNames'][$lastpart], $commaParts);
		}
		elseif (isset($placeOptions['stateAbs'][$lastpart])) {
			#This placename has a state abbreviation
			$newPlaceInfo = formatPlaceName2($placeOptions['stateAbs'][$lastpart], $lastpart, $commaParts);
		}
		else #Non-USA place
			$newPlaceInfo = array('placename'=>$originalPlaceName);
		return $newPlaceInfo;

	} catch (Exception $e) {
		#If there's an exception, return the original placename.
 		echo 'Exception ',$e->getMessage();
		return array('placename'=>$originalPlaceName);
	}
} # end of formatPlaceName1 function

#############################################
# PHP function to do the work of formatting a USA placename.
# Places that consist of only a country have already been dealt with, and the country has been stripped.
# Input parameters:
#  $stateName & $stateAb - The state name and abbreviation for this placename
#  $commaParts - A zero-based array holding the comma-parts of the placename (without a country)
# The return value is an array:
#  ['placename'] = the normalized placename
#  ['citynote'] = a string that was removed from the Placename and is to be saved in an event note.
#  ['unformatted'] = The number of placeparts, if the placename could not be fully normalized,
#                    e.g. for Denver, CO, where Denver could be a county or city, 'unformatted'=2. 
function formatPlaceName2( $stateName, $stateAb, $commaParts) {
	global $placeOptions;
	$unformattedParts=0; #Number of Place-Parts in an improperly-formatted value.
	#$cityNote will be set if a place-part is supposed to be moved to an event note
	$cityNote = ''; 

	if (count($commaParts) > 1) {
		if ($placeOptions['useStateAb']) 
			$commaParts[count($commaParts)-1] = $stateAb;  #Use the state abbreviation
		else $commaParts[count($commaParts)-1] = $stateName; #Use the full state name
	}
	if (count($commaParts) == 1) {
		# For a state by itself, always use the full state name.
		$commaParts[0] = $stateName;
	}
	elseif (count($commaParts) == 2) { ########### 2 parts; just (city or county) and state.
		$countyPart = $commaParts[count($commaParts)-2];
		#If the second part is one of the non-county strings, we can leave it alone.
		if (!isANonCounty($countyPart, $stateAb)) {
			#Well, it's not an official non-county.  We'll have to assume it is a county if the
			#county value contains a county designator, and an ambiguous town/city if there is not.
			$countyName = removeCountyDesignator($countyPart);
			if ($countyName != $countyPart) {
				#We removed a county designator, so add the new county designator.
				$countyDesignator = $placeOptions[$stateab=='LA'?'parish' : 'county']['nocity'];
				$commaParts[count($commaParts)-2] = $countyName . $countyDesignator;
			} else { #This is an ambiguous town/county County position. That is
				#ok in what we call pseudo-states or non-state states, including U.S. 
				#territories, Washington, DC, and other special locations like "East Texas"
				#or "New England", but not in the 50 U.S. states.
				if (!isset($placeOptions['extraStateNamesLookup'][strtoupper($stateName)]))
					#This is a standard USA state; flag the placename
					$unformattedParts = 2; 
			}
		} #end of two-part county/city
	} #end of 2 part location

	elseif (count($commaParts) == 3) { ######### 3 parts; (location or city), county, and state.
		#We have to look at the special cases of the city part to see which  
		#county/parish designator should be added to the county name.
		$countyPart = $commaParts[count($commaParts)-2];
		$cityPart = $commaParts[count($commaParts)-3];
		#If the county part is a non-county, we can leave it alone. (Null counties are non-counties.)
		if (isANonCounty($countyPart, $stateAb)) {
			#This is a non-County. We don't need to deal with the county designator.
			#Non-counties can have cemeteries
			if (isABurialPlace($cityPart)) {
				if ($placeOptions['cemeteryToDo'] == 'move')
					#Move the cemetery to the location position by inserting a blank
					#element at the appropriate position in the array.
					array_splice($commaParts, 1, 0, '');
				elseif ($placeOptions['cemeteryToDo'] == 'remove')
					array_shift($commaParts); #Just throw it away
				elseif ($placeOptions['cemeteryToDo'] == 'note')
					#Save the 'cityNote' as we remove the City Part
					$cityNote = array_shift($commaParts);
				#else 'leave', 'county', 'city', and 'location' are all the same as far as
				#placename formatting goes, and require no action.
			}
		}
		else { #This appears to be a regular county, so we have to deal with the county 
			# designator and all kinds of things in the city part.
			$countyType = $stateab=='LA'?'parish' : 'county';
			$todo = '';
			# Check for city parts that aren't really towns or cities - i.e. cemeteries, 
			# non-cities, and Funky Towns
			if (isABurialPlace($cityPart))
				$todo = $placeOptions['cemeteryToDo'];
			elseif (isANonCity($cityPart))
				$todo = $placeOptions['nonCityToDo'];
			elseif (isAFunkyTown($cityPart))
				$todo = $placeOptions['funkyTownToDo'];
			elseif (($wardtodo=$placeOptions['wardToDo'])!='leave' && preg_match('/ (Ward [\d]*)/i',$cityPart, $matches)) {
				# Modify the city commaPart
				$commaParts[count($commaParts)-3] = str_replace($matches[0], '', $cityPart);
				if ($wardtodo == 'move') { 
					# move "Ward nn" to location, inserting before the city
					$commaParts = array_pad($commaParts, -4, $matches[1]);
				} elseif ($wardtodo == 'note') {
					#Remove the city part and save it to be placed in a note.
					$cityNote = $matches[1];
				}
			}
			if ($todo) {
				if ($todo == 'remove') #Just throw it away
					array_shift($commaParts);
				if ($todo == 'move') #We need to insert an empty city part
					array_splice($commaParts, 1, 0, '');
				if ($todo == 'note')
					#Remove the city part and save it to be placed in a note.
					$cityNote = array_shift($commaParts);
				#else 'leave', 'county', 'city', and 'location' are all the same as far as
				#placename formatting goes, and require no action.
				#Assign a county designator
				$countyDesignator = $placeOptions[$countyType]['nocity'];
				$commaParts[count($commaParts)-2] = removeCountyDesignator($countyPart) . $countyDesignator;
			} else #no special city value=>apply the designator if there is already one.
				$countyDesignator = $placeOptions[$countyType]['city'];
				$commaParts[count($commaParts)-2] = addCountyDesignator($countyPart, $countyDesignator);
			#end if $todo
		} #end is this a non-county or a real county.
	} #end of 3 part locations 

	else { ################### $nparts>= 4; location, city, county, state
		#We can have non-Counties but no special non-cities or funky towns, and can count
		#on cemeteries always being in the location position.
		$countyPart = $commaParts[$nparts-2];
		if (!isANonCounty($countyPart, $stateAb)) {
			#This is a real County, and the city is a real city - if it is present
			$cityPresence = $commaParts[$nparts-3] ? 'city' : 'nocity';
			$countyType = $stateab=='LA'?'parish' : 'county';
			$countyDesignator = $placeOptions[$countyType][$cityPresence];
			$commaParts[$nparts-2] = addCountyDesignator($countyPart, $countyDesignator);
		} #end if non-county or regular county.
	} # End of 4-part location and cascading IF on number of placeparts

	#Put the parts back together in newPlaceName.
	if ($placeOptions['usaTextWith']) 
		array_push($commaParts, $placeOptions['usaTextWith']); #Add the usaText
	$newPlaceName = implode($placeOptions['delimiter'], $commaParts);

	return array('placename' => $newPlaceName, 
		'citynote' => $cityNote,
		'unformatted' => $unformattedParts);
} #End function formatPlaceName2 

#############################################
# Remove a County Designator from the end of a county value, and add the new County Designator.
# Return the new county value.
function addCountyDesignator($countyPart, $newDesignator) {
	if (!$countyPart) return '';
	$countyName = preg_replace('/( County| Co| Co\.| Parish| Par| Par\.)$/', '', $countyPart);
#var_dump($newDesignator, $matches); #[[xyz]]
	if (!$countyName) return '';
	return $countyName . $newDesignator;
} #end function addCountyDesignator

#############################################
# Remove the county designator from the county value.
# We don't have to return a flag that says we removed a county designator, because
# the calling function can just compare the new and old county values.
function removeCountyDesignator($countyPart) {
	if (!$countyPart) return '';
	#Strip ' County ', ' Co.', or ' Co' from the end of the string.
	$countyName = preg_replace('/( County| Co| Co\.| Parish| Par| Par\.)$/', '', $countyPart);
	return $countyName;
} #end function replaceCountyDesignator

#############################################
# PHP function to look at a place part to see if is specifically a cemetery.
# This does not look for non-cities or funky census places.
# We're looking here for substrings, so we have to look at each possible non-city substring
# until we find a match.
function isABurialPlace($placepart) {
	global $placeOptions;
	foreach ($placeOptions['cemeteries'] as $searchkey) {
		if (stripos($placepart, $searchkey) !== false)
			return true;
	}
	return false;
} #end function isABurialPlace

#############################################
# PHP function to look at a place part to see if is specifically a "non-city"
# This does not look for cemeteries or funky census places.
# We're looking here for substrings, so we have to look at each possible non-city substring,
# until we find a match.
function isANonCity($placepart) {
	global $placeOptions;
	foreach ($placeOptions['nonCities'] as $searchkey) {
		if (stripos($placepart, $searchkey) !== false)
			return true;
	}
	return false;
} #end function isANonCity

#############################################
# PHP function to look at a place part to see if is specifically a Funky Census Place
# such as "Civil District 10" or "Magisterial District 17".
# This does not look for non-cities or cemeteries. These strings must match the beginning
# of the placepart.
function isAFunkyTown($placepart) {
	global $placeOptions;
	foreach ($placeOptions['funkyTowns'] as $searchkey) {
		if (stripos($placepart, $searchkey) === 0)
			return true;
	}
	return false;
} #end function isAFunkyTown

#############################################
# PHP function that uses the array of non-county placenames (such as "Long Island,NY" or
# "Shenandoah Vally,VA" to see if the input county/state combination represents a non-county
# placename. The lookup array is an associative array where the subscript is 
# (Geographic Name, State Abbrev)
function isANonCounty($mycounty, $mystate) {
	global $placeOptions;
	if (!$mycounty) return true; #missing counties are "non-counties"
	return isset($placeOptions['nonCounties']["$mycounty,$mystate"]);
} #end function isANonCounty

?>
