<?php
# rrplacenameformat_optionslib.php - Include file installed by the Placename Format mod.
# Contains PHP functions that read the Placename Formatting options, and display those options in a
# form (along with a couple of subforms), plus JavaScript functions that deal with the subforms and
# that use AJAX to save the Placename Formatting options to their config file.
#
# This function library is used by the programs gedcom2tngform.php (installed by the Gedcom 
# Converter mod) and rrplacenameformat_options.php (installed by the Placename Format mod).
$tngModNames['Placename Format']['rrplacenameformat_optionslib.php'] = '4'; # Register the mod 
echo "<style>
	.rrpnf {max-width:62em;} /* Placename formatting div */
	.rrpnf h4 {margin-bottom:0;}
	.rrpnf textarea {padding:3px;}
	.rrpnf li.p {margin-bottom:1em;}
	.rrsavebutton {text-align:right;}
</style>\n";

#############################################
/** Function used to load the global array of Placename Formatting Option ($placeOptions).
*The function argument tells us whether to read them from their config file or to assume that
*they have been passed to this program as global FORM fields - with the same variable names
*1. A set of global variables that come from the configuration files or from form parameters,
*    and that may be used by the Place Options form, which can be presented as part of the
*    Gedcom Converter options, or as a standalone form.
* 2. A set of subscripts to $placeOptions, which is used by the Gedcom Converter, 
*    standalone place normalization (i.e. of the places table), place level assignment,
*    and other programs that look only at the cemetery strings.
*/
function getPlaceNameFormattingParams($source = 'configfile') {
	global $extspath;
	global $placeOptions;
	global $usaPossibleValues, $statescol1, $statescol2, $noncytcol1, $nonctycol2,
		$cemeteryStrings, $nonCityStrings, $funkyTowns, $cemeteryToDo, $nonCityToDo, $funkyTownToDo,
		$usaTextAlone, $usaTextWith, $useStateAb, $countyParishWithCity, $countyParishWithoutCity, $wardToDo,
		$compressPlaces;
	#Non-usa options
	global $canFormatAllPlaces, $levelEmptyCounty, $levelEmptyState, $partsEmptyCounty, $partsEmptyState;

	if ($source == 'configfile') {
		# Include the site-specific config file if it exists. Otherwise, include the distribution version.
		# This config file defines the values used control placename formatting.
		if (file_exists("$extspath/rrplacenameformat_config.php")) include("$extspath/rrplacenameformat_config.php"); 
		else include("$extspath/rrplacenameformat_config-distrib.php");

		#Check for options whose names are different or omitted in old config files
		if (isset($blnUseFullname)) $useStateAb = !$blnUseFullname;
		if (isset($blnCompressPlaces)) $compressPlaces = $blnCompressPlaces;
		#Old configuration files may have use the variables $countyWithCity and $countyWithoutCity,
		#in which case they have omitted the corresponding Parish values.
		if (isset($countyWithoutCity)) {
			$countyParishWithoutCity = computeParishValue($countyWithoutCity);
			$countyParishWithCity = computeParishValue($countyWithCity);
		}
		#Define some values that would be missing in old config files.
		if (!isset($usaPossibleValues)) 
			$usaPossibleValues = 'USA, United States of America, U.S.A., United States';
		if (!isset($funkyTowns)) 
			$funkyTowns = 'Beat, Civil District, Division, Election District, Justice Precinct, Magisterial, No Township, Precinct, Rural';
		if (!isset($cemeteryToDo)) $cemeteryToDo = 'move';
		if (!isset($nonCityToDo)) $nonCityToDo = 'county'; #Placelevel=county
		if (!isset($funkyTownToDo)) $funkyTownToDo = 'county'; #Placelevel=county
		if (!isset($wardToDo)) $wardToDo = 'move'; #Move to location part
if (false) { #xyz no non-usa options yet
	#Non-usa options that may be omitted from the configuration file
		if (!isset($canFormatAllPlaces)) $canFormatAllPlaces = '';
		if (!isset($levelEmptyCounty)) $levelEmptyCounty = '';
		if (!isset($levelEmptyState)) $levelEmptyState = '';
		if (!isset($partsEmptyCounty)) $partsEmptyCounty = '';
		if (!isset($partsEmptyState)) $partsEmptyState = '';
		if (!isset($emptyFormatting)) $emptyFormatting = 0;
		#LevelEmptyCounty & LevelEmptyState must cover all countries one way or another
		if (!$levelEmptyCounty && !$levelEmptyState) $levelEmptyCounty='*';
		#Make sure that either the county or the state is selected if any countries use P
		#???if ($usePlaceLevel && !$omitCounty && !$omitState)
} #xyz no non-Usa options yet
	}
	else { #Get values from options form. These variable are different in the options form
		$extraStateNames = $statescol1;
		$extraStateAbbrvs = $statescol2;
		$nonCountyLocales = $nonctycol1;
		$nonCountyStates = $nonctycol2;
	}

	#Actions that take place wither the Place Options are read from a file as form parameters.
	#Explode the list of possible USA values into uppercase associative array keys so we can 
	#look up the last place-part of an unformatted placename to see if it is a USA place.
	$placeOptions['usaPossibleValuesString'] = $usaPossibleValues;
	$placeOptions['usaPossibleValues'] = explodeOption($usaPossibleValues);

	#The states dictionaries(associative arrays) are also placed in $placeOptions
	$placeOptions['stateNames'] =array(); #State name -> state abbreviations
	$placeOptions['stateAbs']=array();    #State abbreviation -> state name
	loadStates();  #Load the 50 states into the states dictionaries.
	#Save the non-standard states and abbreviations as zero-based numbered arrays, 
	#and add them to the states list.
	$placeOptions['extraStateNames'] = explode("|", $extraStateNames);
	$placeOptions['extraStateAbbrvs'] = explode("|", $extraStateAbbrvs);
	$placeOptions['extraStateNamesLookup'] = explodeOption($extraStateNames, '|');
	# Load the extra states into the states dictionaries.
	for ($i=0; $i<count($placeOptions['extraStateNames']); $i++) {
		saveState(trim($placeOptions['extraStateNames'][$i]), trim($placeOptions['extraStateAbbrvs'][$i]));
	}
	#Expand the vbar-delimited lists of NonCountyLocales and NonCountyStates into zero-based arrays,
	#and into a separate zero-based array whose values are (NonCountyLocale,NonCountyState)
	$placeOptions['nonCountyLocales'] = array_map('trim',explode('|', $nonCountyLocales)); 
	$placeOptions['nonCountyStates'] = array_map('trim',explode('|', $nonCountyStates)); 
	$placeOptions['nonCounties'] = array();
	for ($i = 0; $i<count($placeOptions['nonCountyLocales']); $i++)
		$placeOptions['nonCounties'][$placeOptions['nonCountyLocales'][$i] . ',' . $placeOptions['nonCountyStates'][$i]] = 1;

	#Take 3 lists of strings and store them in placeOptions, then explode them and store
	#the exploded lists as zero-based arrays in placeOptions. Since these are all substrings
	# - not entire place-parts - we can't use an associative array to look them up.
	$placeOptions['cemeteryStrings'] = $cemeteryStrings;
	$placeOptions['nonCityStrings'] = $nonCityStrings;
	$placeOptions['funkyTownsString'] = $funkyTowns;
	$placeOptions['cemeteries'] = array_map('trim', explode(',', $cemeteryStrings));
	$placeOptions['nonCities'] = array_map('trim', explode(',', $nonCityStrings));
	$placeOptions['funkyTowns'] = array_map('trim', explode(',', $funkyTowns));

	#The "What to do if you find one of these strings" options for the three lists
	$placeOptions['cemeteryToDo'] = $cemeteryToDo;
	$placeOptions['nonCityToDo'] = $nonCityToDo;
	$placeOptions['funkyTownToDo'] = $funkyTownToDo;

	$placeOptions['usaTextAlone'] = $usaTextAlone;
	$placeOptions['usaTextWith'] = $usaTextWith;
	$placeOptions['useStateAb'] = $useStateAb;

	# Two options that combine a county designator and a parish designator
	$placeOptions['countyParishWithCity'] = $countyParishWithCity;
	$placeOptions['countyParishWithoutCity'] = $countyParishWithoutCity;
	$tmp = array_map('trim', explode(',', $countyParishWithCity));
	$placeOptions['county']['city'] = $tmp[0] ? (' ' . $tmp[0]) : '';
	$placeOptions['parish']['city'] = $tmp[1] ? (' ' . $tmp[1]) : '';
	$tmp = array_map('trim', explode(',', $countyParishWithoutCity));
	$placeOptions['county']['nocity'] = $tmp[0] ? (' ' . $tmp[0]) : '';
	$placeOptions['parish']['nocity'] = $tmp[1] ? (' ' . $tmp[1]) : '';

	$placeOptions['wardToDo'] = $wardToDo;
	$placeOptions['compressPlaces'] = $compressPlaces;
	#'delimiter' is derived from a saved option
	$placeOptions['delimiter'] = $compressPlaces ? ',' : ', ';

	#Non-USA options - comma-delimited lists of countries
	if (false) { #Not Yet
		$placeOptions['canFormatAllPlaces'] = explodeOption($canFormatAllPlaces);
		$placeOptions['levelEmptyCounty'] = explodeOption($levelEmptyCounty);
		$placeOptions['levelEmptyState'] = explodeOption($levelEmptyState);
		$placeOptions['partsEmptyCounty'] = explodeOption($partsEmptyCounty);
		$placeOptions['partsEmptyState'] = explodeOption($partsEmptyState);
		#And one checkbox
		$placeOptions['emptyFormatting'] = $emptyFormatting;
		#LevelEmptyCounty & LevelEmptyState must cover all countries one way or another
		if (!$levelEmptyCounty && !$levelEmptyState) $levelEmptyCounty='*';
			$placeOptions['levelEmptyCounty'] = '*';
	} #end of Not Yet non-USA options

} #End of getPlaceNameFormattingParams

################################################################
/** Function to explode an option consisting of a comma-delimited list of values into uppercase 
* associative array keys so we can look up the appropriate place part to see if it matches.
* That is, change $stringvar = "cemetery, memorial garden, mausoleum" to
*                 $placeOptions[{subscript}]['cemetery']=1
*                 $placeOptions[{subscript}]['memorial garden']=1, etc
*/
function explodeOption($optionValue, $delimiter=',') {
	$newArray = array();
	$tmp = explode($delimiter, $optionValue);
	foreach ($tmp as $value) {
		if ($value)
			$newArray[strtoupper(trim($value))] = '';
	}
	return $newArray;
} #end function Exploade Option

#############################################
/** This function displays the form content, and does not return any value.
* The function parameter indicates whether the fields go in a subform that can be hidden.
* The form organizes the options into groups (A, B, C), but numbers the options in 
* one sequence, irrespective of group.
*/
function layOutPlaceNameFormattingFormFields() {
	global $admtext, $extspath, $allow_admin, $placeOptions; 

	#The Save Options button at the top of the form.
	#The "placeoptionssaved" element is for the "Options Saved" message returned from Ajax.
	$disabled = $allow_admin ? '' : ' disabled';
	echo "<div class='rrsavebutton'>
		<button type='button' onclick='savePlaceOptions();'$disabled />{$admtext['rrpnf-saveplace']}</button>
		<br><span id='placeoptionssaved'></span>
		</div>\n";
	echo "<div class='rrpnf'>
	<h4 style='margin-top:0;'>A. {$admtext['rrpnf-placenamesa']}</h4>
	<ol>\n"; ##-end of multiline echo

	# Input options, which describe what unformatted places look like:
		#Possible USA values
		echo "<li><label>{$admtext['rrpnf-usaprompta']}<br>
			<textarea name='usaPossibleValues' id='usaPossibleValues' rows='1' cols='60'>{$placeOptions['usaPossibleValuesString']}</textarea></label>
			</li>\n"; 
		# Non-standard state abbreviations
		echo '<li>', twoColumnSubform('states', $placeOptions['extraStateNames'], $placeOptions['extraStateAbbrvs'] ), "</li>\n";
		# Locales that act like counties.
		echo '<li>', twoColumnSubform('noncty', $placeOptions['nonCountyLocales'], $placeOptions['nonCountyStates'] ), "</li>\n";

	#End the initial inputs list; start strings & their disposition
	echo "</ol>
		<h4>B. {$admtext['rrpnf-placenamesb']}</h4>
		<ol start='4'>\n";

		#Cemetery strings
		echo "<li class='p'>a. <label><b>{$admtext['rrpnf-cemeteries']}:</b><br />
			<textarea name='cemeteryStrings' id='cemeteryStrings' xstyle='height:3.3em;' rows='2' cols='60'>{$placeOptions['cemeteryStrings']}</textarea></label>
			<br/>b. <b>{$admtext['rrpnf-cemeterytodo']}</b>\n";
			$checked = $placeOptions['cemeteryToDo']=='location' ? ' checked' : '';
			echo "<br/><label><input type='radio' name='cemeteryToDo' value='location' $checked>{$admtext['rrpnf-location']}</label>\n";
			$checked = $placeOptions['cemeteryToDo']=='remove' ? ' checked' : '';
			echo "<br/><label><input type='radio' name='cemeteryToDo' value='remove'$checked/>{$admtext['rrpnf-remove']}</label>\n";
			$checked = $placeOptions['cemeteryToDo']=='move' ? ' checked' : '';
			echo "<br/><label><input type='radio' name='cemeteryToDo' value='move'$checked>{$admtext['rrpnf-movelocation']}</label>\n";
			$checked = $placeOptions['cemeteryToDo']=='note' ? ' checked' : '';
			echo "<br/><label><input type='radio' name='cemeteryToDo' value='note'$checked/>{$admtext['rrpnf-movenote']}</label>\n";
		echo "</li>\n";
		#Non-cities - strings within counties that aren't towns
		echo "<li class='p'>a. <label><b>{$admtext['rrpnf-noncities']}:</b><br />
			<textarea name='nonCityStrings' id='nonCityStrings' rows='1' cols='60'>{$placeOptions['nonCityStrings']}</textarea></label>
			<br/>b. <b>{$admtext['rrpnf-noncitytodo']}:</b>\n";
			$checked = $placeOptions['nonCityToDo']=='leave' ? ' checked' : '';
			echo "<br/><label><input type='radio' name='nonCityToDo' value='leave' $checked/>{$admtext['rrpnf-leave']}</label>\n";
			$checked = $placeOptions['nonCityToDo']=='location' ? ' checked' : '';
			echo "<br/><label><input type='radio' name='nonCityToDo' value='location' $checked/>{$admtext['rrpnf-location']}</label>\n";
			$checked = $placeOptions['nonCityToDo']=='county' ? ' checked' : '';
			echo "<br/><label><input type='radio' name='nonCityToDo' value='county' $checked/>{$admtext['rrpnf-county']}</label>\n";
			$checked = $placeOptions['nonCityToDo']=='remove' ? ' checked' : '';
			echo "<br/><label><input type='radio' name='nonCityToDo' value='remove'$checked/>{$admtext['rrpnf-remove']}</label>\n";
			$checked = $placeOptions['nonCityToDo']=='move' ? ' checked' : '';
			echo "<br/><label><input type='radio' name='nonCityToDo' value='move' $checked/>{$admtext['rrpnf-movelocation']}</label>\n";
			$checked = $placeOptions['cemeteryToDo']=='note' ? ' checked' : '';
			echo "<br/><label><input type='radio' name='cemeteryToDo' value='note'$checked/>{$admtext['rrpnf-movenote']}</label>\n";
		echo "</li>\n";
		#Funky census towns
		echo "<li class='p'>a. <label><b>{$admtext['rrpnf-funkytodo']}:</b><br />
			<textarea name='funkyTowns' id='funkyTowns' xstyle='height:3.3em;' rows='2' cols='60'>{$placeOptions['funkyTownsString']}</textarea></label>
			<br/>b. <b>What to do with these funky city/town names:</b>\n";
			$checked = $placeOptions['nonCityToDo']=='leave' ? ' checked' : '';
			echo "<br/><label><input type='radio' name='funkyTownToDo' value='leave'$checked/>{$admtext['rrpnf-leave']}</label>\n";
			$checked = $placeOptions['funkyTownToDo']=='remove' ? ' checked' : '';
			echo "<br/><label><input type='radio' name='funkyTownToDo' value='remove'$checked/>{$admtext['rrpnf-remove']}</label>\n";
			$checked = $placeOptions['funkyTownToDo']=='note' ? ' checked' : '';
			echo "<br/><label><input type='radio' name='funkyTownToDo' value='note'$checked/>{$admtext['rrpnf-movenote']}</label>\n";
		echo "</li>\n";

	#End the inputs list; start the outputs list.
	echo "</ol>
		<h4>C. {$admtext['rrpnf-placenamesc']}</h4>
		<ol start='7'>\n";

	# Output options, which describe what is the formatted places should look like:
		#Text to represent USA...
		echo "<li><label>{$admtext['rrpnf-usapromptc1']}: <input type='text' name='usaTextAlone' 
			id='usaTextAlone' size='18' value=\"{$placeOptions['usaTextAlone']}\" /></label></li>\n";
		echo "<li><label>{$admtext['rrpnf-usapromptc2']}: <span class='nw'><input type='text' name='usaTextWith' 
			id='usaTextWith' size='18' value=\"{$placeOptions['usaTextWith']}\" />
			{$admtext['rrpnf-usanote']}</span></label></li>\n";
		#Use Full state name or abbrev
		if ($placeOptions['useStateAb']) { $fullchecked=''; $abbrvchecked='checked'; }
		else { $fullchecked='checked'; $abbrvchecked=''; }
		echo "<li>{$admtext['rrpnf-use']}
			<label><input type='radio' name='useStateAb' value='0' $fullchecked/>{$admtext['rrpnf-fullname']}</label>
			<label><input type='radio' name='useStateAb' value='1' $abbrvchecked/>{$admtext['rrpnf-abbr']}</label></b></li>\n";

		#County & parish designators to use when there IS as city
		echo "<li><label>{$admtext['rrpnf-countyparishwithcity']}</b>
			<input type='text' name='countyParishWithCity' id='countyParishWithCity' value=\"{$placeOptions['countyParishWithCity']}\"/></label></li>\n";

		#County designator to use when there IS NOT as city
		echo "<li><label>{$admtext['rrpnf-countyparishwithoutcity']}</b>
			<input type='text' name='countyParishWithoutCity' id='countyParishWithoutCity' value=\"{$placeOptions['countyParishWithoutCity']}\"/></label></li>\n";

		#Disposition of "Ward"
		echo "<li class='p'>{$admtext['rrpnf-wardtodo']}</b>:\n";
			$checked = $placeOptions['wardToDo']=='remove' ? ' checked' : '';
			echo "<br/><label><input type='radio' name='wardToDo' value='remove'$checked>{$admtext['rrpnf-wardremove']}</label>\n";
			$checked = $placeOptions['wardToDo']=='move' ? ' checked' : '';
			echo "<br><label><input type='radio' name='wardToDo' value='move'$checked>{$admtext['rrpnf-wardmove']}</label>\n";
			$checked = $placeOptions['wardToDo']=='note' ? ' checked' : '';
			echo "<br><label><input type='radio' name='wardToDo' value='note'$checked>{$admtext['rrpnf-wardnote']}</label>\n";
			$checked = $placeOptions['wardToDo']=='leave' ? ' checked' : '';
			echo "<br><label><input type='radio' name='wardToDo' value='leave'$checked>{$admtext['rrpnf-wardleave']}</label>\n";
		echo "</li>\n";

		#Compress places
		$checked = $placeOptions['compressPlaces'] ? ' checked' : '';
		echo "<li><label><input type='checkbox' name='compressPlaces' id='compressPlaces' value='1'$checked />{$admtext['rrpnf-removespaces']}</b></label></li>\n";

if (false) { #xyz don't do non-usa yet
	#End the USA output list; start the non-USA list.
	echo "</ol>
		<h4>D. Non-USA Places</h4>
		Non-USA Places with an embedded empty place-part will always be Formatted and Split based on the Place-parts,
		and other non-USA Plaes with a Place Level will always be Formatted and Split based on the Place Level.
		<ol start='13'>\n";

			echo "<li>For places being split using Place Levels, if there enough Place-parts to populate the State/Province OR the County/Shire, but not both:
				<ol type='a'>\n";
				#Countries where the 'State' position is to be left empty 
				echo "<li><label>Countries where the 'State/Province' place-part is to be left empty:
					<br/><textarea rows='1' cols='80' name='levelEmptyState' id='levelEmptyState'>{$placeOptions['levelEmptyState']}</textarea>
					</label></li>\n";
				#Countries where the 'County' position is to be left empty 
				echo "<li><label>Countries where the 'County/Shire' place-part is to be left empty:
				<br/><textarea rows='1' cols='80' name='levelEmptyCounty' id='levelEmptyCounty'>{$placeOptions['levelEmptyCounty']}</textarea>
				</label></li>
				</ol></li>\n";
			#Countries where any place can be Formatted
			echo "<li><label>Countries where any Place (even those without a Place Level or an embedded empty Place-Part) can be Formatted and Split into Place-Parts:
				<span class='nw'>(Separate with commas; use * for all)</span>:
				<br/><textarea style='height:3.3em;' rows='2' cols='80' name='canFormatAllPlaces' id='canFormatAllPlaces'>{$placeOptions['canFormatAllPlaces']}</textarea>
			</label>\n";
			echo "<li>For places being split based only on Place-parts, without a Place Level or an embedded empty Place-Part:
				<ol type='a'>\n";
				#Countries where the 'State' position is to be left empty 
				echo "<li><label>Countries where the 'State/Province' place-part is always to be left emipty:
					<br/><textarea rows='1' cols='80' name='partsEmptyState' id='partsEmptyState'>{$placeOptions['partsEmptyState']}</textarea>
					</label></li>\n";
				#Countries where the 'County' position is to be left empty 
				echo "<li><label>Countries where the 'County/Shire' place-part is always to be left empty:
				<br/><textarea rows='1' cols='80' name='partsEmptyCounty' id='partsEmptyCounty'>{$placeOptions['partsEmptyCounty']}</textarea>
				</label></li>
				</ol></li>\n";
			#Add empty place-parts when Formatting
			$checked = $placeOptions['emptyFormatting'] ? ' checked' : '';
			echo "<li><label><input type='checkbox' name='emptyFormatting' id='emptyFormatting' value='1'$checked />
				Add empty Place-parts (where called for) when formatting Placenames.</label></li>\n";
} #xyz don't do non-usa yet

		echo "</ol><br/>\n"; #End the options list.
	#A second Save Options button for the bottom of the form. 
	$disabled = $allow_admin ? '' : ' disabled';
	echo "<div class='rrsavebutton'>
		<button type='button' onclick='savePlaceOptions();'$disabled/>{$admtext['rrpnf-saveplace']}</button>
		</div>\n";
	echo "</div>\n"; #End the div that wraps the form fields

} # End of layOutPlaceNameFormattingFormFields PHP function

#############################################
/* This PHP function is called from the Placename form layout function twice to lay 
* out two subforms that each contain two columns and a variable number of rows.
* One subform handles non-standard states: state name & state abbreviation.
* The other handles Locales that Act like Counties: locale & state abbreviation.
* Parameters:
* - $subform - The subform baseID, which is used in the button that opens and 
*    closes the fieldset that wraps the subform, the layout table that holds 
*    the fields, and the name and id of each field.
* - $col1values - array of values to go in column 1
* - $col2values - array of values to go in column 2
*/
function twoColumnSubform($subform, $col1values, $col2values) {
	global $admtext;
	# First, generate 
	# - buttons to open and close the subform, 
	# - a message that notes that closing the subform does not save any changes.
	# - a fieldset to hold the subform, and
	# - two hidden fields that are given initial empty values. When the form is submitted, they
	#   will be populated with vbar-delimited strings that contain all values in a column.
	$html = "<input name='open$subform' id='open$subform' type='button' class='smallbutton' 
			value=\"{$admtext['rrpnf-button'.$subform]}\" onclick=\"toggleTwoColumnSubform('$subform')\"/>
		<span id='close$subform' style='display:none;'>
			<input name='close$subform' type='button' class='smallbutton' 
				value=\"{$admtext['rrpnf-closebutton']}\" onclick=\"toggleTwoColumnSubform('$subform')\"/>
			{$admtext['rrpnf-doesnotsave']}
		</span>
		<input type='hidden' name='{$subform}col1' id='{$subform}col1' value=''>
		<input type='hidden' name='{$subform}col2' id='{$subform}col2' value=''>
		<fieldset id='fieldset$subform' style='width:50em; display:none;'>
			<legend>{$admtext['rrpnf-legend'.$subform]}</legend>\n"; ##end of multi-line assignment statement

			# Place some annotation inside the subform and above the table
			$html .= $admtext['rrpnf-note'.$subform] . "<p>{$admtext['rrpnf-subformnote']}</p>\n";
			#Take the two vbar-delimited variables the two fields, and turn them into arrays.
			#Add a pair of empty values  to generate an empty row.
			$col1values[] = '';
			$col2values[] = '';
			#Start a two-column table with headings for the two fields.
			$html .= "<table id='table$subform' border=0>
				<tr><th>" .$admtext['rrpnf-col1'.$subform] . "</th>
					<th>" .$admtext['rrpnf-col2'.$subform] . "</th></tr>\n";
			#loop through one of the arrays, displaying the two values in each row. 
			#The style class assigned to the table rows is used to count rows in the table.
			for ($i=0; $i<count($col1values); $i++) {
				$html .= "<tr class='row$subform'><td><input type='text' name='col1$subform$i'
						id='col1$subform$i' size='20' value=\"{$col1values[$i]}\"/></td>
					<td><input type='text' name='col2$subform$i'
						id='col2$subform$i' size='10' value=\"{$col2values[$i]}\"/></td></tr>\n";
			}
			#After the table, generate a button that can add a row of fields to the subform.
			$html .= "</table>
			<input type='button' class='smallbutton' value=\"{$admtext['rrpnf-addrow']}\" 
				onclick = \"addRowToTwoColumnSubform('$subform')\"/>
		</fieldset>\n"; ##-End of multi-line assignment statement
	return $html;
} #End of twoColumnSubform PHP function


################################################################
/** PHP function to load the 50 states into associative arrays
* Washington, DC needs to be added as a pseudo-state because it has no counties.
*/
function loadStates() {
	saveState('Alabama', 'AL');
	saveState('Alaska', 'AK');
	saveState('Arizona', 'AZ');
	saveState('Arkansas', 'AR');
	saveState('California', 'CA');
	saveState('Colorado', 'CO');
	saveState('Connecticut', 'CT');
	saveState('Delaware', 'DE');
	saveState('Florida', 'FL');
	saveState('Georgia', 'GA');
	saveState('Hawaii', 'HI');
	saveState('Idaho', 'ID');
	saveState('Illinois', 'IL');
	saveState('Indiana', 'IN');
	saveState('Iowa', 'IA');
	saveState('Kansas', 'KS');
	saveState('Kentucky', 'KY');
	saveState('Louisiana', 'LA');
	saveState('Maine', 'ME');
	saveState('Maryland', 'MD');
	saveState('Massachusetts', 'MA');
	saveState('Michigan', 'MI');
	saveState('Minnesota', 'MN');
	saveState('Mississippi', 'MS');
	saveState('Missouri', 'MO');
	saveState('Montana', 'MT');
	saveState('Nebraska', 'NE');
	saveState('Nevada', 'NV');
	saveState('New Hampshire', 'NH');
	saveState('New Jersey', 'NJ');
	saveState('New Mexico', 'NM');
	saveState('New York', 'NY');
	saveState('North Carolina', 'NC');
	saveState('North Dakota', 'ND');
	saveState('Ohio', 'OH');
	saveState('Oklahoma', 'OK');
	saveState('Oregon', 'OR');
	saveState('Pennsylvania', 'PA');
	saveState('Rhode Island', 'RI');
	saveState('South Carolina', 'SC');
	saveState('South Dakota', 'SD');
	saveState('Tennessee', 'TN');
	saveState('Texas', 'TX');
	saveState('Utah', 'UT');
	saveState('Vermont', 'VT');
	saveState('Virginia', 'VA');
	saveState('Washington', 'WA');
	saveState('West Virginia', 'WV');
	saveState('Wisconsin', 'WI');
	saveState('Wyoming', 'WY');
} #end function loadStates

#############################################################
/** PHP function to load a single state and its abbreviation into associative arrays.
*/
function saveState($stateName,$stAb) {
	global $placeOptions;
	$placeOptions['stateNames'][$stateName] = $stAb;
	$placeOptions['stateAbs'][$stAb] = $stateName;
} #end function saveState

################################################################
# Function to compute a parish designator (really, a county,parish value) based on the county
# designator that was provided by an old config file.
function computeParishValue($county) {
	switch ($county) {
		case 'County' : return 'County,Parish';
		case 'Co' : return 'Co,Par';
		case 'Co.' : return 'Co.,Par.';
		case '': return ',';
		default: return "$county,Parish";
	} #end switch
} #End function computeParishValue

#############################################################
# This JavaScript block contains functions that are used within the Placename Formatting options form.
?>
<script language='JavaScript'>
//Toggle the display state of a subform, the button that opens it, and the button that closes it.
function toggleTwoColumnSubform(subform) {
	$('#fieldset'+subform).toggle();
	$('#open'+subform).toggle();
	$('#close'+subform).toggle();
}

//Add a row to a two-column subform. The parameter is the base ID of the fieldset that contains
//the subform, the table that contains the fields, and the fieldnames and fieldIDs of the fields.
//The fieldset ID is {subformID}fieldset, the table id is {subformID}table, and the field id's are
//{subformID}col1 and {subformID}col2. 
function addRowToTwoColumnSubform(subformID) {
	//Count the rows by counting elements of class "tablecol1" in the subform;
	//i.e. in the fieldset with id=subformID+'fieldset'.
	var i = $('#'+subformID+'fieldset').find('.row'+subformID).length;
	//The fieldname suffixes started with 0 for the first row, so this new row can just use the count.
	var x1="<tr><td><input type='text' id='col1" + i + "' class='tablecol1' size='20' value=''/></td>";
	x1 +="<td><input type='text' id='col2" + i + "' size='10' value=''/></td></tr>";
	$('#table'+subformID+' tr:last').after(x1);
}

// Function called by the Placename Formatting options form 'Submit' button, which is really an action.
// button. It first calls validatePlaceOptions().
// If the validation is successful, it then uses Ajax to invoke the action script that writes the
// form to the options file.
// (We use an action button and Ajax because the Placename Formatting form fields can be on the same
// web page as the Gedcom Converter Options form, and we need to be able to save each set of options
// separately.)
function savePlaceOptions() {
	if (validateAndImplodePlaceOptions()) {
		var params = {
			usaPossibleValues : $('#usaPossibleValues').val(),
			statescol1 : $('#statescol1').val(),
			statescol2 : $('#statescol2').val(),
			nonctycol1 : $('#nonctycol1').val(),
			nonctycol2 : $('#nonctycol2').val(),
			cemeteryStrings : $('#cemeteryStrings').val(),
			cemeteryToDo : $('input[name=cemeteryToDo]:checked').val(),
			nonCityStrings : $('#nonCityStrings').val(),
			nonCityToDo : $('input[name=nonCityToDo]:checked').val(),
			funkyTowns : $('#funkyTowns').val(),
			funkyTownToDo : $('input[name=funkyTownToDo]:checked').val(),
			usaTextAlone : $('#usaTextAlone').val(),
			usaTextWith : $('#usaTextWith').val(),
			useStateAb : $('input[name=useStateAb]:checked').val(),
			countyParishWithCity : $('#countyParishWithCity').val(),
			countyParishWithoutCity : $('#countyParishWithoutCity').val(),
			wardToDo : $('input[name=wardToDo]:checked').val(),
			compressPlaces : ($('#compressPlaces').is(':checked') ? 1 : 0)
		};
		// #xyz temporary variable for nonUSA Options
		var nonUsaOptions = {
			canFormatAllPlaces : $('#canFormatAllPlaces').val(),
			levelEmptyCounty : $('#levelEmptyCounty').val(),
			levelEmptyState : $('#levelEmptyState').val(),
			partsEmptyCounty : $('#partsEmptyCounty').val(),
			partsEmptyState : $('#partsEmptyState').val(),
			emptyFormatting : ($('#emptyFormatting').is(':checked')? 1 : 0)
		};
		$.ajax({
			type: "POST",
			url: 'rrplacenameformat_saveoptions.php',
			data: params,
			dataType: 'html',
			success: function(results){
				//Add the results to the HTML element set aside for results.
				$('#placeoptionssaved').html($('#placeoptionssaved').html() + ' &nbsp;&nbsp ' + results);
			}
		});
	}
} //end of savePlaceOptions

// Function to validate the Placename Format form and pull data out of the two subforms.
function validateAndImplodePlaceOptions() {
	if (!implodeTwoColumnSubform('noncty')) return false;
	return implodeTwoColumnSubform('states');
}

// disableTwoColumnSubform disables the exploded fields that are displayed in the two-column
// form. The function implodeTwoColumnSubform copied all of their values to a pair of fields
// that are passed to the action script. We don't want to see the exploded fields any more.
// The consolidated fields are outside of the subordinate fieldset, so we can just disable
// all fields in the fieldset.
// This function is not called from inside this PHP module. It is called only when a 
// program submits a form that contains the Placename Format fields. 
function disableTwoColumnSubform(subform) {
	$('#fieldset'+subform+' :input').each(function(index) {
		$(this).attr('disabled',true);
	})
}

// implodeTwoColumnSubform consolidates data from a subform into two hidden fields, and
// validates the data while doing so.
// The form parameter (subform) is a string that identifies the subform.
// Each subform is in a fieldset whose id is fieldset{subform}
// Within that fieldset, the form fields are in a variable-length table of two columns of fields.
// The fields in the two columns are named col1{subform}i and col2{subform}i, where i is the row number.
// The fields in column 1 all have the class "subformcol1".
// This function consolidates each column of data into a vbar-delimited hidden fields. 
// The two fields are named {subform}col1 and {subform}col2.
// Validation:  Empty rows are ok, but if there is a value in either column within a row,
// then there must be values in both columns.
function implodeTwoColumnSubform(subform) {
	var col1=""; //The value being built from column 1
	var col2=""; //The value being built from column 2
	var field1,field2; //Temporary variables for the 2 field values in one row
	//To get the number of field pairs, we look at the fieldset element that contains the
	//subform, and its list of elements with class "subformcol1";
	var fields = $('#fieldset'+subform).find('.row'+subform).length;
	for (var i=0; i<fields; i++) {
		field1 = $('#col1'+subform+i).val();
		field2 = $('#col2'+subform+i).val();
		if (field1!="" && field2!="") {
			//Both fields have values.  Store them in the two strings. Don't add a vbar before the first element
			col1 += ((col1!="") ? "|" : "") + field1;
			col2 += ((col2!="") ? "|" : "" )+ field2;
		} else if ((field1!="" && field2=="") || (field1=="" && field2!="")) {
			//Only one of the fields has a value. That's an error
			alert ("<?php echo $admtext['rrpnf-subformpair']; ?>".replace('###', subform));
			return false;
		}
	}
	//Store the two strings in their hidden fields
	$('#'+subform+'col1').val(col1);
	$('#'+subform+'col2').val(col2);
	return true;
} // End of the implodeTwoColumnSubform validation script

</script>
